/*******************************************************************************
* File Name: Comp_Reference.c  
* Version 1.70
*
*  Description:
*    This file provides the source code to the API for the 8-bit Voltage DAC 
*    (VDAC8) User Module.
*
*   Note:
*     Any unusual or non-standard behavior should be noted here. Other-
*     wise, this section should remain blank.
*
*******************************************************************************
* Copyright 2008-2011, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "Comp_Reference.h"

#if (CY_PSOC5_ES1)
#include <CyLib.h>
#endif /* CY_PSOC5_ES1 */

/* Check to see if required defines such as CY_PSOC3 and CY_PSOC5 are available */
/* They are defined starting with cy_boot v2.30 */
#ifndef CY_PSOC3
#error Component VDAC8_v1_70 requires cy_boot v2.30 or later
#endif /* CY_PSOC3 */


uint8 Comp_Reference_initVar = 0;

#if (CY_PSOC5_ES1)
uint8 Comp_Reference_restoreVal = 0;
uint8 Comp_Reference_intrStatus = 0u;
#endif /* CY_PSOC5_ES1 */

static Comp_Reference_backupStruct Comp_Reference_backup;


/*******************************************************************************
* Function Name: Comp_Reference_Init
********************************************************************************
* Summary:
*     Initialize to the schematic state
* 
* Parameters:  
*  void:  
*
* Return: 
*  (void)
*
* Theory: 
*
* Side Effects: 
*
* Reenrancy: 
*  Yes
*
*******************************************************************************/
void Comp_Reference_Init(void) 
{
    Comp_Reference_CR0 = (Comp_Reference_MODE_V )  ;    

    /* Set default data source */
    if(Comp_Reference_DEFAULT_DATA_SRC != 0 )    
    {
        Comp_Reference_CR1 = (Comp_Reference_DEFAULT_CNTL | Comp_Reference_DACBUS_ENABLE) ;
    }
    else
    {
        Comp_Reference_CR1 = (Comp_Reference_DEFAULT_CNTL | Comp_Reference_DACBUS_DISABLE) ;
    } 
    
    /* Set default strobe mode */
    if(Comp_Reference_DEFAULT_STRB != 0)
    {
        Comp_Reference_Strobe |= Comp_Reference_STRB_EN ;
    }
    
    /* Set default range */
    Comp_Reference_SetRange(Comp_Reference_DEFAULT_RANGE); 
    
    /* Set default speed */
    Comp_Reference_SetSpeed(Comp_Reference_DEFAULT_SPEED);
}


/*******************************************************************************
* Function Name: Comp_Reference_Enable
********************************************************************************
* Summary:
*     Enable the VDAC8
* 
* Parameters:  
*  void:  
*
* Return: 
*  (void)
*
* Theory: 
*
* Side Effects: 
*
* Reenrancy: 
*  Yes
*
*******************************************************************************/
void Comp_Reference_Enable(void) 
{
    Comp_Reference_PWRMGR |= Comp_Reference_ACT_PWR_EN;
    Comp_Reference_STBY_PWRMGR |= Comp_Reference_STBY_PWR_EN;

    /*This is to restore the value of register CR0 ,
    which is modified  in Stop API , this prevents misbehaviour of VDAC */

    #if (CY_PSOC5_ES1)
    if ( Comp_Reference_restoreVal == 1) 
    {
         Comp_Reference_CR0 = Comp_Reference_backup.data_value;
         Comp_Reference_restoreVal = 0;
    }
    #endif
}


/*******************************************************************************
* Function Name: Comp_Reference_Start
********************************************************************************
*
* Summary:
*  The start function initializes the voltage DAC with the default values, 
*  and sets the power to the given level.  A power level of 0, is the same as
*  executing the stop function.
*
* Parameters:  
*  Power:   Sets power level between off (0) and (3) high power
*
* Return: 
*  (void) 
*
* Global variables:
*  Comp_Reference_initVar: Is modified when this function is called for the 
*   first time. Is used to ensure that initialization happens only once.
*
* Reenrancy: 
*  No
*
*******************************************************************************/
void Comp_Reference_Start(void) 
{

    /* Hardware initiazation only needs to occure the first time */
    if ( Comp_Reference_initVar == 0)  
    { 
        Comp_Reference_Init();
       
        Comp_Reference_initVar = 1;
    }
    
    /* Enable power to DAC */
    Comp_Reference_Enable();
    
    /* Set default value */
    Comp_Reference_SetValue(Comp_Reference_DEFAULT_DATA); 
}


/*******************************************************************************
* Function Name: Comp_Reference_Stop
********************************************************************************
*
* Summary:
*  Powers down DAC to lowest power state.
*
* Parameters:  
*  (void)  
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
* Reenrancy: 
*  Yes
*
*******************************************************************************/
void Comp_Reference_Stop(void) 
{
   /* Disble power to DAC */
   Comp_Reference_PWRMGR &= ~Comp_Reference_ACT_PWR_EN;
   Comp_Reference_STBY_PWRMGR &= ~Comp_Reference_STBY_PWR_EN;
   
   
    /* This is a work around for PSoC5  ,
    this sets VDAC to current mode with output off */
    
    #if (CY_PSOC5_ES1)
        Comp_Reference_backup.data_value = Comp_Reference_CR0;
        Comp_Reference_CR0 = Comp_Reference_CUR_MODE_OUT_OFF;
        Comp_Reference_restoreVal = 1;
    #endif
    
   
}


/*******************************************************************************
* Function Name: Comp_Reference_SetSpeed
********************************************************************************
*
* Summary:
*  Set DAC speed
*
* Parameters:  
*  power:   Sets speed value 
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
* Reenrancy: 
*  Yes
*
*******************************************************************************/
void Comp_Reference_SetSpeed(uint8 speed) 
{
   /* Clear power mask then write in new value */
   Comp_Reference_CR0 &= ~Comp_Reference_HS_MASK ;    
   Comp_Reference_CR0 |=  ( speed & Comp_Reference_HS_MASK) ;    
}


/*******************************************************************************
* Function Name: Comp_Reference_SetRange
********************************************************************************
*
* Summary:
*  Set one of three current ranges.
*
* Parameters:  
*  Range:  Sets one of Three valid ranges.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
* Reenrancy: 
*  Yes
*
*******************************************************************************/
void Comp_Reference_SetRange(uint8 range) 
{
   Comp_Reference_CR0 &= ~Comp_Reference_RANGE_MASK ;      /* Clear existing mode */
   Comp_Reference_CR0 |= ( range & Comp_Reference_RANGE_MASK ) ; /*  Set Range  */
   Comp_Reference_DacTrim();
}


/*******************************************************************************
* Function Name: Comp_Reference_SetValue
********************************************************************************
*
* Summary:
*  Set 8-bit DAC value
*
* Parameters:  
*  value:  Sets DAC value between 0 and 255.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
* Reenrancy: 
*  Yes
*******************************************************************************/
void Comp_Reference_SetValue(uint8 value) 
{

   #if (CY_PSOC5_ES1)
       Comp_Reference_intrStatus = CyEnterCriticalSection();
   #endif /* CY_PSOC5_ES1 */
   
   Comp_Reference_Data = value;                             /*  Set Value  */
   
   /* PSOC3 silicons earlier to ES3 require a double write */
   #if (CY_PSOC3_ES2 || CY_PSOC5_ES1)
       Comp_Reference_Data = value;
   #endif /* CY_PSOC3_ES2 || CY_PSOC5_ES1 */
   
   #if (CY_PSOC5_ES1)
       CyExitCriticalSection(Comp_Reference_intrStatus);
   #endif /* CY_PSOC5_ES1 */
   
}


/*******************************************************************************
* Function Name: Comp_Reference_DacTrim
********************************************************************************
*
* Summary:
*  Set the trim value for the given range.
*
* Parameters:  
*   range:  1V or 4V range.  See constants.  
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
* Reenrancy: 
*  Yes
*
*******************************************************************************/
void Comp_Reference_DacTrim(void) 
{
    uint8 mode;
    
    mode = ((Comp_Reference_CR0 & Comp_Reference_RANGE_MASK) >> 2) + Comp_Reference_TRIM_M7_1V_RNG_OFFSET;
   Comp_Reference_TR = CY_GET_XTND_REG8((uint8 *)(Comp_Reference_DAC_TRIM_BASE + mode));
}


/* [] END OF FILE */
