/*******************************************************************************
* File Name: Output_drive.c  
* Version 1.70
*
*  Description:
*    This file provides the source code to the API for the 8-bit Voltage DAC 
*    (VDAC8) User Module.
*
*   Note:
*     Any unusual or non-standard behavior should be noted here. Other-
*     wise, this section should remain blank.
*
*******************************************************************************
* Copyright 2008-2011, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "Output_drive.h"

#if (CY_PSOC5_ES1)
#include <CyLib.h>
#endif /* CY_PSOC5_ES1 */

/* Check to see if required defines such as CY_PSOC3 and CY_PSOC5 are available */
/* They are defined starting with cy_boot v2.30 */
#ifndef CY_PSOC3
#error Component VDAC8_v1_70 requires cy_boot v2.30 or later
#endif /* CY_PSOC3 */


uint8 Output_drive_initVar = 0;

#if (CY_PSOC5_ES1)
uint8 Output_drive_restoreVal = 0;
uint8 Output_drive_intrStatus = 0u;
#endif /* CY_PSOC5_ES1 */

static Output_drive_backupStruct Output_drive_backup;


/*******************************************************************************
* Function Name: Output_drive_Init
********************************************************************************
* Summary:
*     Initialize to the schematic state
* 
* Parameters:  
*  void:  
*
* Return: 
*  (void)
*
* Theory: 
*
* Side Effects: 
*
* Reenrancy: 
*  Yes
*
*******************************************************************************/
void Output_drive_Init(void) 
{
    Output_drive_CR0 = (Output_drive_MODE_V )  ;    

    /* Set default data source */
    if(Output_drive_DEFAULT_DATA_SRC != 0 )    
    {
        Output_drive_CR1 = (Output_drive_DEFAULT_CNTL | Output_drive_DACBUS_ENABLE) ;
    }
    else
    {
        Output_drive_CR1 = (Output_drive_DEFAULT_CNTL | Output_drive_DACBUS_DISABLE) ;
    } 
    
    /* Set default strobe mode */
    if(Output_drive_DEFAULT_STRB != 0)
    {
        Output_drive_Strobe |= Output_drive_STRB_EN ;
    }
    
    /* Set default range */
    Output_drive_SetRange(Output_drive_DEFAULT_RANGE); 
    
    /* Set default speed */
    Output_drive_SetSpeed(Output_drive_DEFAULT_SPEED);
}


/*******************************************************************************
* Function Name: Output_drive_Enable
********************************************************************************
* Summary:
*     Enable the VDAC8
* 
* Parameters:  
*  void:  
*
* Return: 
*  (void)
*
* Theory: 
*
* Side Effects: 
*
* Reenrancy: 
*  Yes
*
*******************************************************************************/
void Output_drive_Enable(void) 
{
    Output_drive_PWRMGR |= Output_drive_ACT_PWR_EN;
    Output_drive_STBY_PWRMGR |= Output_drive_STBY_PWR_EN;

    /*This is to restore the value of register CR0 ,
    which is modified  in Stop API , this prevents misbehaviour of VDAC */

    #if (CY_PSOC5_ES1)
    if ( Output_drive_restoreVal == 1) 
    {
         Output_drive_CR0 = Output_drive_backup.data_value;
         Output_drive_restoreVal = 0;
    }
    #endif
}


/*******************************************************************************
* Function Name: Output_drive_Start
********************************************************************************
*
* Summary:
*  The start function initializes the voltage DAC with the default values, 
*  and sets the power to the given level.  A power level of 0, is the same as
*  executing the stop function.
*
* Parameters:  
*  Power:   Sets power level between off (0) and (3) high power
*
* Return: 
*  (void) 
*
* Global variables:
*  Output_drive_initVar: Is modified when this function is called for the 
*   first time. Is used to ensure that initialization happens only once.
*
* Reenrancy: 
*  No
*
*******************************************************************************/
void Output_drive_Start(void) 
{

    /* Hardware initiazation only needs to occure the first time */
    if ( Output_drive_initVar == 0)  
    { 
        Output_drive_Init();
       
        Output_drive_initVar = 1;
    }
    
    /* Enable power to DAC */
    Output_drive_Enable();
    
    /* Set default value */
    Output_drive_SetValue(Output_drive_DEFAULT_DATA); 
}


/*******************************************************************************
* Function Name: Output_drive_Stop
********************************************************************************
*
* Summary:
*  Powers down DAC to lowest power state.
*
* Parameters:  
*  (void)  
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
* Reenrancy: 
*  Yes
*
*******************************************************************************/
void Output_drive_Stop(void) 
{
   /* Disble power to DAC */
   Output_drive_PWRMGR &= ~Output_drive_ACT_PWR_EN;
   Output_drive_STBY_PWRMGR &= ~Output_drive_STBY_PWR_EN;
   
   
    /* This is a work around for PSoC5  ,
    this sets VDAC to current mode with output off */
    
    #if (CY_PSOC5_ES1)
        Output_drive_backup.data_value = Output_drive_CR0;
        Output_drive_CR0 = Output_drive_CUR_MODE_OUT_OFF;
        Output_drive_restoreVal = 1;
    #endif
    
   
}


/*******************************************************************************
* Function Name: Output_drive_SetSpeed
********************************************************************************
*
* Summary:
*  Set DAC speed
*
* Parameters:  
*  power:   Sets speed value 
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
* Reenrancy: 
*  Yes
*
*******************************************************************************/
void Output_drive_SetSpeed(uint8 speed) 
{
   /* Clear power mask then write in new value */
   Output_drive_CR0 &= ~Output_drive_HS_MASK ;    
   Output_drive_CR0 |=  ( speed & Output_drive_HS_MASK) ;    
}


/*******************************************************************************
* Function Name: Output_drive_SetRange
********************************************************************************
*
* Summary:
*  Set one of three current ranges.
*
* Parameters:  
*  Range:  Sets one of Three valid ranges.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
* Reenrancy: 
*  Yes
*
*******************************************************************************/
void Output_drive_SetRange(uint8 range) 
{
   Output_drive_CR0 &= ~Output_drive_RANGE_MASK ;      /* Clear existing mode */
   Output_drive_CR0 |= ( range & Output_drive_RANGE_MASK ) ; /*  Set Range  */
   Output_drive_DacTrim();
}


/*******************************************************************************
* Function Name: Output_drive_SetValue
********************************************************************************
*
* Summary:
*  Set 8-bit DAC value
*
* Parameters:  
*  value:  Sets DAC value between 0 and 255.
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
* Reenrancy: 
*  Yes
*******************************************************************************/
void Output_drive_SetValue(uint8 value) 
{

   #if (CY_PSOC5_ES1)
       Output_drive_intrStatus = CyEnterCriticalSection();
   #endif /* CY_PSOC5_ES1 */
   
   Output_drive_Data = value;                             /*  Set Value  */
   
   /* PSOC3 silicons earlier to ES3 require a double write */
   #if (CY_PSOC3_ES2 || CY_PSOC5_ES1)
       Output_drive_Data = value;
   #endif /* CY_PSOC3_ES2 || CY_PSOC5_ES1 */
   
   #if (CY_PSOC5_ES1)
       CyExitCriticalSection(Output_drive_intrStatus);
   #endif /* CY_PSOC5_ES1 */
   
}


/*******************************************************************************
* Function Name: Output_drive_DacTrim
********************************************************************************
*
* Summary:
*  Set the trim value for the given range.
*
* Parameters:  
*   range:  1V or 4V range.  See constants.  
*
* Return: 
*  (void) 
*
* Theory: 
*
* Side Effects:
*
* Reenrancy: 
*  Yes
*
*******************************************************************************/
void Output_drive_DacTrim(void) 
{
    uint8 mode;
    
    mode = ((Output_drive_CR0 & Output_drive_RANGE_MASK) >> 2) + Output_drive_TRIM_M7_1V_RNG_OFFSET;
   Output_drive_TR = CY_GET_XTND_REG8((uint8 *)(Output_drive_DAC_TRIM_BASE + mode));
}


/* [] END OF FILE */
