/*******************************************************************************
* File Name: SCP_Timeout.c  
* Version 2.10
*
* Description:
*  The PWM User Module consist of an 8 or 16-bit counter with two 8 or 16-bit
*  comparitors. Each instance of this user module is capable of generating
*  two PWM outputs with the same period. The pulse width is selectable between
*  1 and 255/65535. The period is selectable between 2 and 255/65536 clocks. 
*  The compare value output may be configured to be active when the present 
*  counter is less than or less than/equal to the compare value.
*  A terminal count output is also provided. It generates a pulse one clock
*  width wide when the counter is equal to zero.
*
* Note:
*
*******************************************************************************
* Copyright 2008-2011, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "cytypes.h"
#include "SCP_Timeout.h"

uint8 SCP_Timeout_initVar = 0u;

/*******************************************************************************
* Function Name: SCP_Timeout_Start
********************************************************************************
*
* Summary:
*  The start function initializes the pwm with the default values, the 
*  enables the counter to begin counting.  It does not enable interrupts,
*  the EnableInt command should be called if interrupt generation is required.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  SCP_Timeout_initVar: Is modified when this function is called for the first 
*   time. Is used to ensure that initialization happens only once.
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SCP_Timeout_Start(void) 
{
    /* If not Initialized then initialize all required hardware and software */
    if(SCP_Timeout_initVar == 0u)
    {
        SCP_Timeout_Init();
        SCP_Timeout_initVar = 1u;
    }
    SCP_Timeout_Enable();

}


/*******************************************************************************
* Function Name: SCP_Timeout_Init
********************************************************************************
*
* Summary:
*  Initialize component's parameters to the parameters set by user in the 
*  customizer of the component placed onto schematic. Usually called in 
*  SCP_Timeout_Start().
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SCP_Timeout_Init(void) 
{
    #if (SCP_Timeout_UsingFixedFunction || SCP_Timeout_UseControl)
        uint8 ctrl;
    #endif
    
    #if(!SCP_Timeout_UsingFixedFunction) 
        #if(SCP_Timeout_UseStatus)
            /* Interrupt State Backup for Critical Region*/
            uint8 SCP_Timeout_interruptState;
        #endif
    #endif
    
   #if (SCP_Timeout_UsingFixedFunction)
        /* You are allowed to write the compare value (FF only) */
        SCP_Timeout_CONTROL |= SCP_Timeout_CFG0_MODE;
        #if (SCP_Timeout_DeadBand2_4)
            SCP_Timeout_CONTROL |= SCP_Timeout_CFG0_DB;
        #endif
                
        /* Set the default Compare Mode */
        #if(SCP_Timeout_PSOC3_ES2 || SCP_Timeout_PSOC5_ES1)
                ctrl = SCP_Timeout_CONTROL2 & ~SCP_Timeout_CTRL_CMPMODE1_MASK;
                SCP_Timeout_CONTROL2 = ctrl | SCP_Timeout_DEFAULT_COMPARE1_MODE;
        #endif
        #if(SCP_Timeout_PSOC3_ES3 || SCP_Timeout_PSOC5_ES2)
                ctrl = SCP_Timeout_CONTROL3 & ~SCP_Timeout_CTRL_CMPMODE1_MASK;
                SCP_Timeout_CONTROL3 = ctrl | SCP_Timeout_DEFAULT_COMPARE1_MODE;
        #endif
        
         /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
        SCP_Timeout_RT1 &= ~SCP_Timeout_RT1_MASK;
        SCP_Timeout_RT1 |= SCP_Timeout_SYNC;     
                
        /*Enable DSI Sync all all inputs of the PWM*/
        SCP_Timeout_RT1 &= ~(SCP_Timeout_SYNCDSI_MASK);
        SCP_Timeout_RT1 |= SCP_Timeout_SYNCDSI_EN;
       
    #elif (SCP_Timeout_UseControl)
        /* Set the default compare mode defined in the parameter */
        ctrl = SCP_Timeout_CONTROL & ~SCP_Timeout_CTRL_CMPMODE2_MASK & ~SCP_Timeout_CTRL_CMPMODE1_MASK;
        SCP_Timeout_CONTROL = ctrl | SCP_Timeout_DEFAULT_COMPARE2_MODE | SCP_Timeout_DEFAULT_COMPARE1_MODE;
    #endif 
        
    #if (!SCP_Timeout_UsingFixedFunction)
        #if (SCP_Timeout_Resolution == 8)
            /* Set FIFO 0 to 1 byte register for period*/
            SCP_Timeout_AUX_CONTROLDP0 |= (SCP_Timeout_AUX_CTRL_FIFO0_CLR);
        #else /* (SCP_Timeout_Resolution == 16)*/
            /* Set FIFO 0 to 1 byte register for period */
            SCP_Timeout_AUX_CONTROLDP0 |= (SCP_Timeout_AUX_CTRL_FIFO0_CLR);
            SCP_Timeout_AUX_CONTROLDP1 |= (SCP_Timeout_AUX_CTRL_FIFO0_CLR);
        #endif
    #endif
        
    SCP_Timeout_WritePeriod(SCP_Timeout_INIT_PERIOD_VALUE);
    SCP_Timeout_WriteCounter(SCP_Timeout_INIT_PERIOD_VALUE);
        
        #if (SCP_Timeout_UseOneCompareMode)
            SCP_Timeout_WriteCompare(SCP_Timeout_INIT_COMPARE_VALUE1);
        #else
            SCP_Timeout_WriteCompare1(SCP_Timeout_INIT_COMPARE_VALUE1);
            SCP_Timeout_WriteCompare2(SCP_Timeout_INIT_COMPARE_VALUE2);
        #endif
        
        #if (SCP_Timeout_KillModeMinTime)
            SCP_Timeout_WriteKillTime(SCP_Timeout_MinimumKillTime);
        #endif
        
        #if (SCP_Timeout_DeadBandUsed)
            SCP_Timeout_WriteDeadTime(SCP_Timeout_INIT_DEAD_TIME);
        #endif

    #if (SCP_Timeout_UseStatus || SCP_Timeout_UsingFixedFunction)
        SCP_Timeout_SetInterruptMode(SCP_Timeout_INIT_INTERRUPTS_MODE);
    #endif
        
    #if (SCP_Timeout_UsingFixedFunction)
        /* Globally Enable the Fixed Function Block chosen */
        SCP_Timeout_GLOBAL_ENABLE |= SCP_Timeout_BLOCK_EN_MASK;
        /* Set the Interrupt source to come from the status register */
        SCP_Timeout_CONTROL2 |= SCP_Timeout_CTRL2_IRQ_SEL;
    #else
        #if(SCP_Timeout_UseStatus)
            
            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            SCP_Timeout_interruptState = CyEnterCriticalSection();
            /* Use the interrupt output of the status register for IRQ output */
            SCP_Timeout_STATUS_AUX_CTRL |= SCP_Timeout_STATUS_ACTL_INT_EN_MASK;
            
             /* Exit Critical Region*/
            CyExitCriticalSection(SCP_Timeout_interruptState);
            
            /* Clear the FIFO to enable the SCP_Timeout_STATUS_FIFOFULL
                   bit to be set on FIFO full. */
            SCP_Timeout_ClearFIFO();
        #endif
    #endif
}


/*******************************************************************************
* Function Name: SCP_Timeout_Enable
********************************************************************************
*
* Summary: 
*  Enables the PWM block operation
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Side Effects: 
*  This works only if software enable mode is chosen
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SCP_Timeout_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (SCP_Timeout_UsingFixedFunction)
        SCP_Timeout_GLOBAL_ENABLE |= SCP_Timeout_BLOCK_EN_MASK;
        SCP_Timeout_GLOBAL_STBY_ENABLE |= SCP_Timeout_BLOCK_STBY_EN_MASK;
    #endif 
    
    /* Enable the PWM from the control register  */
    #if (SCP_Timeout_UseControl || SCP_Timeout_UsingFixedFunction)
        SCP_Timeout_CONTROL |= SCP_Timeout_CTRL_ENABLE;
    #endif
}


/*******************************************************************************
* Function Name: SCP_Timeout_Stop
********************************************************************************
*
* Summary:
*  The stop function halts the PWM, but does not change any modes or disable
*  interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects:
*  If the Enable mode is set to Hardware only then this function
*  has no effect on the operation of the PWM
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SCP_Timeout_Stop(void) 
{
    #if (SCP_Timeout_UseControl || SCP_Timeout_UsingFixedFunction)
        SCP_Timeout_CONTROL &= ~SCP_Timeout_CTRL_ENABLE;
    #endif
    
    /* Globally disable the Fixed Function Block chosen */
    #if (SCP_Timeout_UsingFixedFunction)
        SCP_Timeout_GLOBAL_ENABLE &= ~SCP_Timeout_BLOCK_EN_MASK;
        SCP_Timeout_GLOBAL_STBY_ENABLE &= ~SCP_Timeout_BLOCK_STBY_EN_MASK;
    #endif
}


#if (SCP_Timeout_UseOneCompareMode)
#if (SCP_Timeout_CompareMode1SW)
/*******************************************************************************
* Function Name: SCP_Timeout_SetCompareMode
********************************************************************************
* 
* Summary:
*  This function writes the Compare Mode for the pwm output when in Dither mode,
*  Center Align Mode or One Output Mode.
*
* Parameters:
*  comparemode:  The new compare mode for the PWM output. Use the compare types
*                defined in the H file as input arguments.
*
* Return:
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SCP_Timeout_SetCompareMode(uint8 comparemode) 
{
    #if(SCP_Timeout_UsingFixedFunction)
            #if(SCP_Timeout_PSOC3_ES2 || SCP_Timeout_PSOC5_ES1)
                        uint8 comparemodemasked = (comparemode << SCP_Timeout_CTRL_CMPMODE1_SHIFT);
                        SCP_Timeout_CONTROL2 &= ~SCP_Timeout_CTRL_CMPMODE1_MASK; /*Clear Existing Data */
                        SCP_Timeout_CONTROL2 |= comparemodemasked;  
                #endif
                
            #if(SCP_Timeout_PSOC3_ES3 || SCP_Timeout_PSOC5_ES2)
                    uint8 comparemodemasked = (comparemode << SCP_Timeout_CTRL_CMPMODE1_SHIFT);
            SCP_Timeout_CONTROL3 &= ~SCP_Timeout_CTRL_CMPMODE1_MASK; /*Clear Existing Data */
            SCP_Timeout_CONTROL3 |= comparemodemasked;     
                #endif
                
    #elif (SCP_Timeout_UseControl)
        uint8 comparemode1masked = (comparemode << SCP_Timeout_CTRL_CMPMODE1_SHIFT) & SCP_Timeout_CTRL_CMPMODE1_MASK;
        uint8 comparemode2masked = (comparemode << SCP_Timeout_CTRL_CMPMODE2_SHIFT) & SCP_Timeout_CTRL_CMPMODE2_MASK;
        SCP_Timeout_CONTROL &= ~(SCP_Timeout_CTRL_CMPMODE1_MASK | SCP_Timeout_CTRL_CMPMODE2_MASK); /*Clear existing mode */
        SCP_Timeout_CONTROL |= (comparemode1masked | comparemode2masked);
        
    #else
        uint8 temp = comparemode;
    #endif
}
#endif /* SCP_Timeout_CompareMode1SW */

#else /* UseOneCompareMode */


#if (SCP_Timeout_CompareMode1SW)
/*******************************************************************************
* Function Name: SCP_Timeout_SetCompareMode1
********************************************************************************
* 
* Summary:
*  This function writes the Compare Mode for the pwm or pwm1 output
*
* Parameters:  
*  comparemode:  The new compare mode for the PWM output. Use the compare types
*                defined in the H file as input arguments.
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SCP_Timeout_SetCompareMode1(uint8 comparemode) 
{
    uint8 comparemodemasked = (comparemode << SCP_Timeout_CTRL_CMPMODE1_SHIFT) & SCP_Timeout_CTRL_CMPMODE1_MASK;
    #if(SCP_Timeout_UsingFixedFunction)
            #if(SCP_Timeout_PSOC3_ES2 || SCP_Timeout_PSOC5_ES1)
                        SCP_Timeout_CONTROL2 &= SCP_Timeout_CTRL_CMPMODE1_MASK; /*Clear existing mode */
                        SCP_Timeout_CONTROL2 |= comparemodemasked; 
            #endif
                
                #if(SCP_Timeout_PSOC3_ES3 || SCP_Timeout_PSOC5_ES2)
                    SCP_Timeout_CONTROL3 &= SCP_Timeout_CTRL_CMPMODE1_MASK; /*Clear existing mode */
                        SCP_Timeout_CONTROL3 |= comparemodemasked; 
            #endif
                
    #elif (SCP_Timeout_UseControl)
        SCP_Timeout_CONTROL &= SCP_Timeout_CTRL_CMPMODE1_MASK; /*Clear existing mode */
        SCP_Timeout_CONTROL |= comparemodemasked;
    #endif    
}
#endif /* SCP_Timeout_CompareMode1SW */


#if (SCP_Timeout_CompareMode2SW)
/*******************************************************************************
* Function Name: SCP_Timeout_SetCompareMode2
********************************************************************************
* 
* Summary:
*  This function writes the Compare Mode for the pwm or pwm2 output
*
* Parameters:  
*  comparemode:  The new compare mode for the PWM output. Use the compare types
*                defined in the H file as input arguments.
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SCP_Timeout_SetCompareMode2(uint8 comparemode) 
{
    #if(SCP_Timeout_UsingFixedFunction)
        /* Do Nothing because there is no second Compare Mode Register in FF block*/ 
    #elif (SCP_Timeout_UseControl)
    uint8 comparemodemasked = (comparemode << SCP_Timeout_CTRL_CMPMODE2_SHIFT) & SCP_Timeout_CTRL_CMPMODE2_MASK;
    SCP_Timeout_CONTROL &= SCP_Timeout_CTRL_CMPMODE2_MASK; /*Clear existing mode */
    SCP_Timeout_CONTROL |= comparemodemasked;
    #endif    
}
#endif /*SCP_Timeout_CompareMode2SW */
#endif /* UseOneCompareMode */


/*******************************************************************************
* Function Name: SCP_Timeout_WriteCounter
********************************************************************************
* 
* Summary:
*  This function is used to change the counter value.
*
* Parameters:  
*  counter:  This value may be between 1 and (2^Resolution)-1.   
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SCP_Timeout_WriteCounter(uint8 counter) 
{
    #if(SCP_Timeout_UsingFixedFunction)
        CY_SET_REG16(SCP_Timeout_COUNTER_LSB_PTR, (uint16)counter);
    #else
        CY_SET_REG8(SCP_Timeout_COUNTER_LSB_PTR, counter);
    #endif
}


#if (!SCP_Timeout_UsingFixedFunction)
/*******************************************************************************
* Function Name: SCP_Timeout_ReadCounter
********************************************************************************
* 
* Summary:
*  This function returns the current value of the counter.  It doesn't matter
*  if the counter is enabled or running.
*
* Parameters:  
*  void  
*
* Return: 
*  The current value of the counter.
*
* Reentrant:
*  Yes
*
*******************************************************************************/
uint8 SCP_Timeout_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    CY_GET_REG8(SCP_Timeout_COUNTERCAP_LSB_PTR);
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    return (CY_GET_REG8(SCP_Timeout_CAPTURE_LSB_PTR));
}


#if (SCP_Timeout_UseStatus)
/*******************************************************************************
* Function Name: SCP_Timeout_ClearFIFO
********************************************************************************
* 
* Summary:
*  This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SCP_Timeout_ClearFIFO(void) 
{
    while(SCP_Timeout_ReadStatusRegister() & SCP_Timeout_STATUS_FIFONEMPTY)
        SCP_Timeout_ReadCapture();
}
#endif /* SCP_Timeout_UseStatus */
#endif /* !SCP_Timeout_UsingFixedFunction */


/*******************************************************************************
* Function Name: SCP_Timeout_WritePeriod
********************************************************************************
* 
* Summary:
*  This function is used to change the period of the counter.  The new period 
*  will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period:  Period value. May be between 1 and (2^Resolution)-1.  A value of 0 
*           will result in the counter remaining at zero.
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SCP_Timeout_WritePeriod(uint8 period) 
{
    #if(SCP_Timeout_UsingFixedFunction)
        CY_SET_REG16(SCP_Timeout_PERIOD_LSB_PTR, (uint16)period);
    #else
        CY_SET_REG8(SCP_Timeout_PERIOD_LSB_PTR, period);
    #endif
}


#if (SCP_Timeout_UseOneCompareMode)
/*******************************************************************************
* Function Name: SCP_Timeout_WriteCompare
********************************************************************************
* 
* Summary:
*  This funtion is used to change the compare1 value when the PWM is in Dither
*  mode. The compare output will reflect the new value on the next UDB clock. 
*  The compare output will be driven high when the present counter value is 
*  compared to the compare value based on the compare mode defined in 
*  Dither Mode.
*
* Parameters:  
*  compare:  New compare value.  
*
* Return: 
*  void
*
* Side Effects:
*  This function is only available if the PWM mode parameter is set to
*  Dither Mode, Center Aligned Mode or One Output Mode
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SCP_Timeout_WriteCompare(uint8 compare) 
{
   CY_SET_REG8(SCP_Timeout_COMPARE1_LSB_PTR, compare);
   #if (SCP_Timeout_PWMMode == SCP_Timeout__B_PWM__DITHER)
        CY_SET_REG8(SCP_Timeout_COMPARE2_LSB_PTR, compare+1);
   #endif
}


#else


/*******************************************************************************
* Function Name: SCP_Timeout_WriteCompare1
********************************************************************************
* 
* Summary:
*  This funtion is used to change the compare1 value.  The compare output will 
*  reflect the new value on the next UDB clock.  The compare output will be 
*  driven high when the present counter value is less than or less than or 
*  equal to the compare register, depending on the mode.
*
* Parameters:  
*  compare:  New compare value.  
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SCP_Timeout_WriteCompare1(uint8 compare) 
{
    #if(SCP_Timeout_UsingFixedFunction)
        CY_SET_REG16(SCP_Timeout_COMPARE1_LSB_PTR, (uint16)compare);
    #else
        CY_SET_REG8(SCP_Timeout_COMPARE1_LSB_PTR, compare);
    #endif
}


/*******************************************************************************
* Function Name: SCP_Timeout_WriteCompare2
********************************************************************************
* 
* Summary:
*  This funtion is used to change the compare value, for compare1 output.  
*  The compare output will reflect the new value on the next UDB clock.  
*  The compare output will be driven high when the present counter value is 
*  less than or less than or equal to the compare register, depending on the 
*  mode.
*
* Parameters:  
*  compare:  New compare value.  
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SCP_Timeout_WriteCompare2(uint8 compare) 
{
    #if(SCP_Timeout_UsingFixedFunction)
        CY_SET_REG16(SCP_Timeout_COMPARE2_LSB_PTR, compare);
    #else
        CY_SET_REG8(SCP_Timeout_COMPARE2_LSB_PTR, compare);
    #endif
}
#endif /* UseOneCompareMode */


#if (SCP_Timeout_DeadBandUsed)
/*******************************************************************************
* Function Name: SCP_Timeout_WriteDeadTime
********************************************************************************
* 
* Summary:
*  This function writes the dead-band counts to the corresponding register
*
* Parameters:  
*  deadtime:  Number of counts for dead time 
*
* Return: 
*  void
*
* Reentrant:
*  Yes
*
*******************************************************************************/
void SCP_Timeout_WriteDeadTime(uint8 deadtime) 
{
    /* If using the Dead Band 1-255 mode then just write the register */
    #if(!SCP_Timeout_DeadBand2_4)
        CY_SET_REG8(SCP_Timeout_DEADBAND_COUNT_PTR, deadtime);
    #else
        /* Otherwise the data has to be masked and offset */        
        /* Clear existing data */
        SCP_Timeout_DEADBAND_COUNT &= ~SCP_Timeout_DEADBAND_COUNT_MASK; 
            /* Set new dead time */
        SCP_Timeout_DEADBAND_COUNT |= (deadtime << SCP_Timeout_DEADBAND_COUNT_SHIFT) & SCP_Timeout_DEADBAND_COUNT_MASK; 
    #endif
}


/*******************************************************************************
* Function Name: SCP_Timeout_ReadDeadTime
********************************************************************************
* 
* Summary:
*  This function reads the dead-band counts from the corresponding register
*
* Parameters:  
*  void
*
* Return: 
*  Dead Band Counts
*
* Reentrant:
*  Yes
*
*******************************************************************************/
uint8 SCP_Timeout_ReadDeadTime(void) 
{
    /* If using the Dead Band 1-255 mode then just read the register */
    #if(!SCP_Timeout_DeadBand2_4)
        return (CY_GET_REG8(SCP_Timeout_DEADBAND_COUNT_PTR));
    #else
        /* Otherwise the data has to be masked and offset */
        return ((SCP_Timeout_DEADBAND_COUNT & SCP_Timeout_DEADBAND_COUNT_MASK) >> SCP_Timeout_DEADBAND_COUNT_SHIFT);
    #endif
}
#endif /* DeadBandUsed */

